package mrtjp.projectred.core;

import mrtjp.projectred.ProjectRedCore;
import mrtjp.projectred.core.ItemPart.EnumPart;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.block.Block;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.FurnaceRecipes;
import net.minecraftforge.oredict.OreDictionary;
import net.minecraftforge.oredict.ShapedOreRecipe;
import net.minecraftforge.oredict.ShapelessOreRecipe;
import codechicken.microblock.ItemMicroPart;
import codechicken.microblock.handler.MicroblockProxy;
import cpw.mods.fml.common.registry.GameRegistry;

public class CoreRecipes
{
    public static void initCoreRecipes()
    {
        initPartRecipes();
        initToolRecipes();
    }

    private static void initToolRecipes()
    {
        /** Draw Plate **/
        GameRegistry.addRecipe(new ShapedOreNBTRecipe(
                new ItemStack(ProjectRedCore.itemDrawPlate(), 1),
                " i ",
                "idi",
                " i ",
                'i', ItemMicroPart.create(770, Blocks.iron_block.getUnlocalizedName()),
                'd', ItemMicroPart.create(2, Blocks.diamond_block.getUnlocalizedName())
                ).setCheckNBT());

        /** Screw Driver **/
        GameRegistry.addRecipe(new ShapedOreRecipe(new ItemStack(ProjectRedCore.itemScrewdriver()),
                "i  ",
                " ib",
                " bi",
                'i', Items.iron_ingot,
                'b', PRColors.BLUE.getOreDict()
                ));

        /** Wire debugger **/
        GameRegistry.addRecipe(new ShapedOreRecipe(new ItemStack(ProjectRedCore.itemWireDebugger()),
                "a a",
                "ber",
                "bgr",
                'a', EnumPart.REDINGOT.getItemStack(),
                'b', PRColors.BLACK.getOreDict(),
                'e', Items.emerald,
                'r', PRColors.RED.getOreDict(),
                'g', Items.glowstone_dust
                ));

        /** Data Card **/
        GameRegistry.addRecipe(new ItemStack(ProjectRedCore.itemDataCard()),
                "pp ",
                "prp",
                "prp",
                'p', Items.paper,
                'r', Items.redstone
                );


    }

    private static void initPartRecipes()
    {
        /** Circuit Plate **/
        GameRegistry.addSmelting(Blocks.stone, EnumPart.PLATE.getItemStack(2), 0f);

        /** Conductive Plate **/
        GameRegistry.addRecipe(EnumPart.CONDUCTIVEPLATE.getItemStack(),
                "r",
                "p",
                'r', Items.redstone,
                'p', EnumPart.PLATE.getItemStack()
                );

        /** Anode **/
        GameRegistry.addRecipe(EnumPart.ANODE.getItemStack(3),
                " r ",
                "rrr",
                "ppp",
                'r', Items.redstone,
                'p', EnumPart.PLATE.getItemStack()
                );

        /** Cathode **/
        GameRegistry.addRecipe(EnumPart.CATHODE.getItemStack(),
                "t",
                "p",
                't', Blocks.redstone_torch,
                'p', EnumPart.PLATE.getItemStack()
                );

        /** Pointer **/
        GameRegistry.addRecipe(EnumPart.POINTER.getItemStack(),
                "b",
                "m",
                "c",
                'b', Blocks.stone,
                'm', Blocks.redstone_torch,
                'c', EnumPart.PLATE.getItemStack()
                );

        /** Silicon Chip **/
        GameRegistry.addRecipe(EnumPart.SILICONCHIP.getItemStack(),
                " s ",
                "ppp",
                's', EnumPart.INFUSEDSILICON.getItemStack(),
                'p', EnumPart.PLATE.getItemStack()
                );

        /** Energized Silicon Chip **/
        GameRegistry.addRecipe(EnumPart.ENERGIZEDSILICONCHIP.getItemStack(),
                " e ",
                "ppp",
                'e', EnumPart.ENERGIZEDSILICON.getItemStack(),
                'p', EnumPart.PLATE.getItemStack()
                );

        /** Platformed Plate **/
        GameRegistry.addRecipe(EnumPart.PLATFORMEDPLATE.getItemStack(),
                " r ",
                "sps",
                "prp",
                'r', EnumPart.WIREDPLATE.getItemStack(),
                's', Items.stick,
                'p', EnumPart.PLATE.getItemStack()
                );


        /** Silicon Boule **/
        addMetaSmelting(ProjectRedCore.itemComponent(), EnumPart.SANDYCOALCOMPOUND.meta, EnumPart.SILICONBOULE.getItemStack(), 0);

        /** Silicon **/
        GameRegistry.addRecipe(EnumPart.SILICON.getItemStack(8),
                "s",
                "b",
                's', new ItemStack(MicroblockProxy.sawDiamond(), 1, OreDictionary.WILDCARD_VALUE),
                'b', EnumPart.SILICONBOULE.getItemStack()
                );

        /** Infused Silicon **/
        addMetaSmelting(ProjectRedCore.itemComponent(), EnumPart.REDSILICONCOMPOUND.meta, EnumPart.INFUSEDSILICON.getItemStack(), 0);

        /** Energized Silicon **/
        addMetaSmelting(ProjectRedCore.itemComponent(), EnumPart.GLOWINGSILICONCOMPOUND.meta, EnumPart.ENERGIZEDSILICON.getItemStack(), 0);

        /** Motor **/
        GameRegistry.addRecipe(EnumPart.MOTOR.getItemStack(),
                " i ",
                "scs",
                "rcr",
                'i', Items.iron_ingot,
                's', Blocks.stone,
                'c', EnumPart.COPPERCOIL.getItemStack(),
                'r', Items.redstone
                );

        /** Copper Coil **/
        GameRegistry.addRecipe(new ShapedOreRecipe(EnumPart.COPPERCOIL.getItemStack(),
                "cd",
                'c', "ingotCopper",
                'd', new ItemStack(ProjectRedCore.itemDrawPlate(), 1, OreDictionary.WILDCARD_VALUE)
                ));

        /** Iron Coil **/
        GameRegistry.addRecipe(EnumPart.IRONCOIL.getItemStack(),
                "cd",
                'c', new ItemStack(Items.iron_ingot),
                'd', new ItemStack(ProjectRedCore.itemDrawPlate(), 1, OreDictionary.WILDCARD_VALUE)
                );

        /** Gold Coil **/
        GameRegistry.addRecipe(EnumPart.GOLDCOIL.getItemStack(),
                "cd",
                'c', new ItemStack(Items.gold_ingot),
                'd', new ItemStack(ProjectRedCore.itemDrawPlate(), 1, OreDictionary.WILDCARD_VALUE)
                );

        /** Red Alloy Ingot **/
        addMetaSmelting(ProjectRedCore.itemComponent(), EnumPart.REDIRONCOMPOUND.meta, EnumPart.REDINGOT.getItemStack(), 0);

        /** Illumar **/
        for (int i = 0; i < EnumPart.ILLUMAR_PARTS.length; i++) {
            EnumPart p = EnumPart.ILLUMAR_PARTS[i];
            GameRegistry.addRecipe(new ShapelessOreRecipe(p.getItemStack(),
                    new ItemStack(Items.glowstone_dust),
                    new ItemStack(Items.glowstone_dust),
                    PRColors.get(i).getOreDict(),
                    PRColors.get(i).getOreDict()
                    ));
        }

        /** Woven Cloth **/
        GameRegistry.addRecipe(EnumPart.WOVENCLOTH.getItemStack(),
                "sss",
                "sws",
                "sss",
                's', Items.string,
                'w', Items.stick
                );

        /** Sail **/
        GameRegistry.addRecipe(EnumPart.SAIL.getItemStack(),
                "ss",
                "ss",
                "ss",
                's', EnumPart.WOVENCLOTH.getItemStack()
                );

        /** Red Iron Compound **/
        GameRegistry.addRecipe(EnumPart.REDIRONCOMPOUND.getItemStack(),
                "rrr",
                "rir",
                "rrr",
                'r', Items.redstone,
                'i', Items.iron_ingot
                );

        /** Sandy Coal Compound **/
        GameRegistry.addRecipe(EnumPart.SANDYCOALCOMPOUND.getItemStack(),
                "sss",
                "scs",
                "sss",
                'c', Blocks.coal_block,
                's', Blocks.sand
                );

        /** Red Silicon Compound **/
        GameRegistry.addRecipe(EnumPart.REDSILICONCOMPOUND.getItemStack(),
                "rrr",
                "rsr",
                "rrr",
                'r', Items.redstone,
                's', EnumPart.SILICON.getItemStack()
                );

        /** Glowing Silicon Compound **/
        GameRegistry.addRecipe(EnumPart.GLOWINGSILICONCOMPOUND.getItemStack(),
                "ggg",
                "gsg",
                "ggg",
                'g', Items.glowstone_dust,
                's', EnumPart.SILICON.getItemStack()
                );

    }
    
    public static void addMetaSmelting(Block input, int meta, ItemStack output, float experience) {
        ItemStack inStack = new ItemStack(input, 1, meta);
        FurnaceRecipes.smelting().func_151394_a(inStack, output, experience);
    }

    public static void addMetaSmelting(Item input, int meta, ItemStack output, float experience) {
        ItemStack inStack = new ItemStack(input, 1, meta);
        FurnaceRecipes.smelting().func_151394_a(inStack, output, experience);
    }

}
