package mrtjp.projectred.exploration;

import java.util.Random;

import net.minecraft.init.Blocks;
import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.client.renderer.texture.IIconRegister;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.world.World;
import gcewing.codechicken.lib.vec.BlockCoord;

/**
 * Special version of mossy cobblestone, that spreads to nearby blocks when
 * under proper conditions.
 * 
 * @author MrTJP
 * 
 */
public class BlockPhotosyntheticCobblestone extends Block
{
    public BlockPhotosyntheticCobblestone()
    {
        super(Material.rock);
        setHardness(2.0F);
        setResistance(10.0F);
        setStepSound(soundTypePiston);
        setBlockName("stoneMoss");
        setCreativeTab(CreativeTabs.tabBlock);
        setBlockTextureName("cobblestone_mossy");
        setTickRandomly(true);
    }

    @Override
    public void updateTick(World w, int x, int y, int z, Random ran)
    {
        if (!w.isAirBlock(x, y + 1, z) || w.canBlockSeeTheSky(x, y + 1, z))
            return;

        for (int i = 0; i < 6; i++)
        {
            BlockCoord bc = new BlockCoord(x, y, z).offset(i);
            Block block = w.getBlock(bc.x, bc.y, bc.z);
            int meta = w.getBlockMetadata(bc.x, bc.y, bc.z);
            if (!w.isAirBlock(bc.x, bc.y + 1, bc.z) || w.canBlockSeeTheSky(bc.x, bc.y + 1, bc.z))
                continue;

            if (block == Blocks.cobblestone)
            {
                if (isBlockWet(w, bc))
                    if (ran.nextInt(3) == 0)
                        w.setBlock(bc.x, bc.y, bc.z, Blocks.mossy_cobblestone, 0, 3);
            }
            else if (block == Blocks.stonebrick && meta == 2)
                if (isBlockWet(w, bc))
                    if (ran.nextInt(3) == 0)
                        w.setBlock(bc.x, bc.y, bc.z, Blocks.stonebrick, 1, 3);
        }
    }

    public boolean isBlockWet(World w, BlockCoord b)
    {
        for (int i = 0; i < 6; i++)
        {
            BlockCoord bc = b.copy().offset(i);
            Block block = w.getBlock(bc.x, bc.y, bc.z);
            if (block == Blocks.flowing_water || block == Blocks.water)
                return true;
        }
        return false;
    }

    @Override
    public void registerBlockIcons(IIconRegister reg)
    {
        super.registerBlockIcons(reg);
        //Block.cobblestoneMossy.registerIcons(reg); //?
    }
}
