package mrtjp.projectred.illumination;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import codechicken.multipart.IRedstoneConnectorBlock;
import mrtjp.projectred.ProjectRedIllumination;
import mrtjp.projectred.core.BasicUtils;
import net.minecraft.init.Blocks;
import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.client.renderer.texture.IIconRegister;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.EnumCreatureType;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.IIcon;
import net.minecraft.util.MovingObjectPosition;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import gcewing.codechicken.lib.vec.BlockCoord;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import cpw.mods.fml.common.registry.GameRegistry;

public class BlockLamp extends Block implements IRedstoneConnectorBlock
{
    public static IIcon[] onIcons = new IIcon[16];
    public static IIcon[] offIcons = new IIcon[16];

    public BlockLamp(String name)
    {
        super(new Material(Material.circuits.getMaterialMapColor()));
        setBlockBounds(0.0F, 0.0F, 0.0F, 1.0F, 1.0F, 1.0F);
        setHardness(0.5F);
        setBlockName(name);
        setCreativeTab(ProjectRedIllumination.tabLighting());
        GameRegistry.registerBlock(this, ItemBlockLamp.class, name);
    }

    @Override
    public void onNeighborBlockChange(World world, int x, int y, int z, Block block)
    {
        TileLamp tile = BasicUtils.getTileEntity(world, new BlockCoord(x, y, z), TileLamp.class);
        if (tile != null)
            tile.onNeighborBlockChange();
    }

    @Override
    public void updateTick(World world, int x, int y, int z, Random par5Random)
    {
        TileLamp tile = BasicUtils.getTileEntity(world, new BlockCoord(x, y, z), TileLamp.class);
        if (tile != null)
            tile.onTick();
    }

    @Override
    public void onBlockAdded(World world, int x, int y, int z)
    {
        onNeighborBlockChange(world, x, y, z, Blocks.air);
    }

    @Override
    public boolean renderAsNormalBlock()
    {
        return true;
    }

    @Override
    public int getRenderType()
    {
        return 0;
    }

    @Override
    public boolean isOpaqueCube()
    {
        return true;
    }

    @Override
    public boolean isNormalCube(IBlockAccess world, int x, int y, int z)
    {
        return true;
    }

    @Override
    public void getSubBlocks(Item item, CreativeTabs tab, List list)
    {
        for (int i = 0; i < 32; i++)
            list.add(new ItemStack(item, 1, i));
    }

    @Override
    public int getLightValue(IBlockAccess world, int x, int y, int z)
    {
        TileLamp tile = BasicUtils.getTileEntity(world, new BlockCoord(x, y, z), TileLamp.class);
        if (tile != null)
            return tile.getLightValue();
        return 0;
    }

    @Override
    public boolean canCreatureSpawn(EnumCreatureType type, IBlockAccess world, int x, int y, int z)
    {
        return false;
    }

    @Override
    public boolean canConnectRedstone(IBlockAccess world, int x, int y, int z, int side)
    {
        return true;
    }

    @Override
    public boolean canProvidePower()
    {
        return true;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void registerBlockIcons(IIconRegister reg)
    {
        for (int i = 0; i < 16; i++)
        {
            onIcons[i] = reg.registerIcon("projectred:lights/lampon/" + i);
            offIcons[i] = reg.registerIcon("projectred:lights/lampoff/" + i);
        }
    }

    @Override
    public ItemStack getPickBlock(MovingObjectPosition target, World world, int x, int y, int z)
    {
        TileLamp tile = BasicUtils.getTileEntity(world, new BlockCoord(x, y, z), TileLamp.class);
        if (tile != null)
            return tile.getDroppedBlock();
        return null;
    }

    @Override
    public boolean removedByPlayer(World world, EntityPlayer player, int x, int y, int z)
    {
        TileLamp tile = BasicUtils.getTileEntity(world, new BlockCoord(x, y, z), TileLamp.class);
        if (tile != null && !player.capabilities.isCreativeMode)
            BasicUtils.dropItem(world, x, y, z, tile.getDroppedBlock());
        return super.removedByPlayer(world, player, x, y, z);
    }

    @Override
    public ArrayList<ItemStack> getDrops(World world, int x, int y, int z, int metadata, int fortune)
    {
        return new ArrayList<ItemStack>(); // Handled on removeBlockByPlayer
    }

    @Override
    public IIcon getIcon(IBlockAccess world, int x, int y, int z, int side)
    {
        TileLamp tile = BasicUtils.getTileEntity(world, new BlockCoord(x, y, z), TileLamp.class);
        if (tile != null)
            if (tile.getLightValue() == 15)
                return onIcons[tile.colour()];
            else
                return offIcons[tile.colour()];
        return super.getIcon(world, x, y, z, side);
    }

    @Override
    public IIcon getIcon(int side, int meta)
    {
        if (meta > 15)
            return onIcons[meta - 16];
        else
            return offIcons[meta];
    }

    @Override
    public TileEntity createTileEntity(World world, int meta)
    {
        TileLamp t = new TileLamp();
        t.prepairPlacement(meta > 15, meta > 15 ? meta - 16 : meta);
        return t;
    }

    @Override
    public boolean hasTileEntity(int meta)
    {
        return true;
    }

    @Override
    public int getConnectionMask(IBlockAccess world, int x, int y, int z, int side)
    {
        return 0x1F;
    }

    @Override
    public int weakPowerLevel(IBlockAccess world, int x, int y, int z, int side, int mask)
    {
        return 0;
    }
}
